# ---- SETUP STEPS ---- #
    Set-Location $HOME
    New-Item -ItemType Directory -Name class
    Set-Location class
    Write-Host "Hello World" > hello.txt
    Write-Host "Some Code" > hello2.ps1
    1..4 | ForEach-Object {Write-Output "Hello $_" > file$_.txt}

#---- Object Types
    # Returns INT32 Object
        42 | Get-Member
    # Returns Double Object
        42.0 | Get-Member
    # Returns STRING Object
        "42" | Get-Member

#---- Viewing object structure - Get-Member
    
    # Display all of the properties and methods of the 'Get-Process' PowerShell object.
        Get-ChildItem | Get-Member
    # Display only the properties of the 
        Get-ChildItem | Get-Member -MemberType properties

#---- Selecting parts of object - Select-Object
    
    # Show all the parts of the file 'hello.txt'
        Get-ChildItem .\hello.txt | Select-Object *
        Get-ChildItem .\hello.txt | Select-Object CreationTime, FullName, Length
    # Show First X Items
        Get-ChildItem | Select-Object -First 1
    # Show Last X Items
        Get-ChildItem | Select-Object -Last 2
    # Show the Xth item (starting with index 0)
        Get-ChildItem | Select-Object -Index 1

#---- Removing objects from the pipeline - Where-Object
    
    # Display only the objects whose value is less then 3
        1..4 | Where-Object {$_ -lt 3}
    # Display files with the number 2 in them.
        Get-ChildItem | Where-Object {$_.Name -like "*2*" }
        Get-ChildItem | Where-Object {$_.Name -match "2" -and $_.name -like "*.sh"}

#---- Sorting objects

    # Display sorted directory contents name
        Get-ChildItem | Sort-Object -Property Name
    # Display sorted directory contents by the 'Last Write Time UTC' in Reverse Order
        Get-ChildItem | Sort-Object -Property LastWriteTimeUtc -Descending

#---- Repeating a task for multiple objects - ForEach-Object

    # Divides each INT object by 1024
        30000, 56798, 12432 | ForEach-Object -Process {$_/1024}

    # Display the name and size of the file divided by 10
        Get-ChildItem | ForEach-Object -Process {$_.Name; $_.Length / 10; " " }


#---- Using Variables, static classes, and methods

    # Set an object to a variable
        $Directory = Get-ChildItem -Path ..\ -Directory
        $Directory | Where-Object {$_.Name -match "class"} | Get-Member *
        $Directory | Where-Object {$_.Name -match "class"} | Format-List -Property *
        $Directory.GetType()

    <#---- .Net Class Member Notation
        To call a static member put the class name in square brackets and use the double colon before the member name.
        Windows PowerShell is flexible enough, that you can write Windows PowerShell code as if it were C#, VB.NET, or even as if it were VBScript or Perl
    #>
    # Use a static Class to do Maths.
        [System.Math] | Get-Member -Static
        [System.Math]::Sqrt(9)
        [System.Math]::PI

    # Using Parenthesis allows you to access object methods and properties
        (Get-ChildItem | Select-Object -Index 0).fullname
        (Get-ChildItem | Select-Object -Index 0).Name
        (Get-Content .\hello.txt).count
        
#---- Getting WMI objects - Get-CimInstance
    <#
        Windows Management Instrumentation (WMI) is a core technology for Windows system administration because it 
        exposes a wide range of information in a uniform manner. Because of how much WMI makes possible, 
        the PowerShell cmdlet for accessing WMI objects, Get-CimInstance, is one of the most useful for doing real work.
    #> 
    Get-CimInstance -Class Win32_ComputerSystem
    Get-CimInstance -Class Win32_AccountSID